from datetime import datetime

from nltk.corpus import stopwords
from nltk.tokenize import word_tokenize
from pymongo import MongoClient
from random import randrange
import os
from UtilsGit import UtilsGit
from pathlib import Path
import re
import string


##  Classe utils.
# Permet de stocker les fonctions utilitaires "transversales"
#


class Utils:
    statdb = None
    chemin = "/home/passerat/Stage/Squelette/"

    # chemin = "/home/jeremie/Stage/Squelette/"

    ##   Filtrage de la saisie utilisateur pour le choix d'un auteur
    # @param Le numéro d'auteur choisi
    @staticmethod
    def switch_auteur(argument):
        switcher = {
            1: "Auteur 1",
            2: "Auteur 2",
            3: "Auteur 3",
        }
        return switcher.get(argument, "Auteur inconnu")

    ##
    #    Connexion avec la base de données
    #    recherche -> find and find_one
    #     insertion -> insert_one and insert_many
    #     maj -> update_one, update_many and replace_one
    #     suppression -> delete_one and delete_many
    @staticmethod
    def initialise_mongo():
        client = MongoClient('localhost', 27017)

        ## extraction d'une base de données. Si elle n'existe pas, elle esst créée
        Utils.statdb = client.squelette

    ## Création d'utilisateur
    # @param nomUtilisateur Le nom de l'utilisateur
    @staticmethod
    def ajouter_utilisateur(nomUtilisateur):
        Utils.statdb.utilisateur.insert_one(
            {"nom": nomUtilisateur, "note": randrange(0, 10, 1), "tiers": []})

    ## Vérification de la présence d'un utilisateur en base
    # @param nomUtilisateurLe nom de l'utilisateur
    # @return True si Utilisateur présent, False sinon
    @staticmethod
    def presence_utilisateur(nomUtilisateur):
        # print("Je suis dans présence utilisateur pour ", nomUtilisateur)
        nmbre = Utils.statdb.utilisateur.find({"nom": nomUtilisateur})
        # print("Et j'en ai trouvé : ", nmbre.count())

        return nmbre.count() == 1

    ##    Creation d'un contenu
    #   @param nomFichier le nom du fichier importé
    #   @param auteur l'auteur qui importe le fichier
    @staticmethod
    def ajouter_contenu(nomFichier, auteur):

        max_id = Utils.get_max_id_contenus()
        if max_id is None:
            max_id = 0

        #print("nomFichier : ", Utils.chemin + "dossier" + nomFichier + "/")

        cheminFichier = Utils.chemin + "dossier" + nomFichier + "/"

        # Un contenu doit être dans un répertoire isolé pour pouvoir le manipuler en versionnage
        Path(cheminFichier).mkdir(parents=True, exist_ok=True)
        os.system("cp '" + Utils.chemin + nomFichier + "'  '" + cheminFichier + nomFichier + "'")

        commit = UtilsGit.commit(cheminFichier, nomFichier, "Version 1")
        #print("commit : ", commit)

        Utils.statdb.contenu.insert_one(
            {"id": max_id + 1, "auteur": auteur, "note": randrange(0, 10, 1),
             "fichier": "dossier" + nomFichier + "/" + nomFichier,
             "versionEnCours": {"numero": 1, "date": datetime.now().strftime("%d/%m/%y"),
                                "popularite": 0, "commit": commit},
             "tags_banque_fixes": ["tag1", "tag2", "tag3", "tag4", "tag5"],
             "tags_banque_libres": ["tagl1", "tagl2", "tagl3", "tagl4", "tagl5"], "type": " ", "source": " "})

        print("Le contenu a bien été ajouté pour l'utilisateur ", auteur)

    ##  Consultation de tous les contenus publics :
    #    Si panier est égal à True, on sort les contenus de l'auteur
    #    Si panier est égal à False, on sort toutes les questions sauf celles de l'auteur (et ses tiers)
    #    (Nb: La problématique des contenus "en cours de modification" sera traitée ultérieurement)
    # @return les contenus de l'auteur ou tous les autres

    @staticmethod
    def get_contenus(auteur, panier=False):
        if panier:
            return Utils.statdb.contenu.find({"auteur": auteur})
        else:
            contenus_tiers = Utils.get_contenus_tiers(auteur)
            tiers_auteur = list()
            for tier in contenus_tiers:
                tiers_auteur.append(tier[0]["contenu"])
            #return Utils.statdb.contenu.find({"auteur": {"$ne": auteur}} or {"id": {"$nin": tiers_auteur}})
            #return Utils.statdb.contenu.find({"auteur": {"$ne": auteur}})
            # print ("tiers de l'auteur : ", tiers_auteur)
            return Utils.statdb.contenu.find({"$and" : [{"auteur": {"$ne": auteur}}, {"id": {"$nin": tiers_auteur}}]})

    ##    Consultation de tous les contenus, sans distinction
    #      (utile temporairement pour la création d'évaluations)
    # @return tous les contenus présents

    @staticmethod
    def get_tous_les_contenus():
        return Utils.statdb.contenu.find()

    ## Suppression d'un contenu (dissociation avec son auteur)
    # @param numContenu l'id du contenu à dissocier

    @staticmethod
    def renier_contenu(numContenu):
        Utils.statdb.contenu.update({"id": numContenu}, {"$set": {"auteur": "Aucun"}})

    ## Suppression d'un contenu (définitive)
    # @param numContenu l'id du contenu à supprimer

    @staticmethod
    def supprimer_contenu(numContenu):
        Utils.statdb.contenu.delete_one({"id": numContenu})

    ## Permet d'associer un contenu avec le panier d'un auteur tiers
    # @param idContenu l'id associé au contenu
    # @param auteur le nom de l'auteur qui désire un favori

    @staticmethod
    def mettre_contenu_en_tiers(idContenu, auteur):
        # print("Le contenu est dans la version ", Utils.getVersionContenuavecId(idContenu))
        Utils.statdb.utilisateur.update({"nom": auteur}, {
            "$push": {
                "tiers": [{"contenu": int(idContenu)}, {"version": Utils.get_version_contenu_avec_id(idContenu)}]}})

    ## Permet de dissocier un contenu du panier d'un auteur tiers
    # Si cet auteur était le seul à posséder une question "anonyme", elle est supprimée
    # @param idContenu l'identifiant du contenu favori
    @staticmethod
    def supprimer_tiers(idContenu, auteur, version):
        Utils.statdb.utilisateur.update({"nom": auteur},
                                        {"$pull": {"tiers": [{"contenu": idContenu}, {"version": version}]}})

    ## Permet de savoir si un contenu possède un auteur légitime
    # @param idContenu l'identifiant du contenu
    @staticmethod
    def is_anonyme(idContenu):
        #print("Is anonyme : ", Utils.statdb.contenu.find_one({"id": idContenu})["auteur"])
        return Utils.statdb.contenu.find_one({"id": idContenu})["auteur"] == "Aucun"

    ## Récupérer un contenu à partir de son identifiant et sa version
    # @param idContenu l'id associé au contenu
    # @param version version désirée
    # @return Le contenu lié à l'id

    @staticmethod
    def get_contenu_avec_id_et_version(idContenu, version):

        # a décommenter quand git sera actif
        Utils.changer_version(idContenu, version, "")
        return Utils.statdb.contenu.find_one({"id": idContenu})
        # return Utils.statdb.contenu.find_one({"$and": [
        #     {"id": idContenu},
        #     {"$or": [{"versionEnCours.numero": version}, {"historique.numero": version}]}
        # ]
        # })

    ## récupérer la version courante d'un contenu
    # @param idContenu l'id du contenu concerné
    @staticmethod
    def get_version_courante(idContenu):
        return Utils.statdb.contenu.find_one({"id": idContenu})["versionEnCours"]["numero"]

    ## Récupérer un contenu à partir de son identifiant
    # @param idContenu l'id associé au contenu
    # @return Le contenu lié à l'id
    @staticmethod
    def get_contenu_avec_Id(idContenu):
        # Id unique donc find_one utilisable
        return Utils.statdb.contenu.find_one({"id": idContenu})

    ## Récupérer la version en cours d'un contenu à partir de son identifiant
    # @param idContenu l'id associé au contenu
    # @return La version en cours du contenu lié à l'id
    @staticmethod
    def get_version_contenu_avec_id(idContenu):
        # print("Je veux le contenu : ", int(idContenu))
        return int(Utils.get_contenu_avec_Id(int(idContenu))["versionEnCours"]["numero"])

    ##  Récupérer l'id maximal (pour les contenus)
    # permet de s'assurer que les id de contenu soient uniques

    @staticmethod
    def get_max_id_contenus():
        ## Fonction permettant de récupérer l'id max présent en base (pour pouvoir incrémenter un nouveau contenu).
        element = Utils.statdb.contenu.find().sort("id", -1).limit(1)

        for elt in element:
            # print("id : ", elt["id"])
            return int(elt["id"])

    ##  Récupérer l'id maximal (pour les evaluations)
    # permet de s'assurer que les id de contenu soient uniques

    @staticmethod
    def get_max_id_evaluations():
        ## Fonction permettant de récupérer l'id max présent en base (pour pouvoir incrémenter un nouveau contenu).
        element = Utils.statdb.evaluation.find().sort("idEval", -1).limit(1)

        for elt in element:
            # print("id : ", elt["id"])
            return int(elt["idEval"])

    ##  Récupérer l'id maximal (pour les contenus en cours de modification)
    # permet de s'assurer que les id de contenu soient uniques

    @staticmethod
    def get_max_id_contenus_en_cours():
        ## Fonction permettant de récupérer l'id max présent en base (pour pouvoir incrémenter un nouveau contenu).
        element = Utils.statdb.encours.find().sort("idContenuEC", -1).limit(1)

        for elt in element:
            #print("id : ", elt)
            return int(elt["idContenuEC"])

    ##  Récupérer l'id maximal (pour les demandes de publication)
    # permet de s'assurer que les id de demande de publication soient uniques

    @staticmethod
    def get_max_id_demandes_de_publication():
        ## Fonction permettant de récupérer l'id max présent en base (pour pouvoir incrémenter un nouveau contenu).
        element = Utils.statdb.demandesPub.find().sort("id", -1).limit(1)

        for elt in element:
            #print("id : ", elt)
            return int(elt["id"])

    ## Récupérer les tiers d'un utilisateur
    # @param auteur le nom de l'utilisateur
    # @return les tiers de l'utilisateur 'auteur'

    @staticmethod
    def get_contenus_tiers(auteur):
        return Utils.statdb.utilisateur.find_one({"nom": auteur})["tiers"]

    ##  Créer une évaluation
    #   @param contenus les numeros des contenus associés à l'évaluation
    #   @auteur l'auteur qui créée l'évaluation

    @staticmethod
    def creer_evaluation(contenus, auteur):

        # création de l'évaluation
        max_id_eval = Utils.get_max_id_evaluations()
        if max_id_eval is None:
            max_id_eval = 0

        Utils.statdb.evaluation.insert_one(
            {"idEval": max_id_eval + 1, "date": datetime.now().strftime("%d/%m/%y"), "auteur": auteur,
             "contenu(s)": contenus})
        # on renseigne l'évaluation dans les contenus associés
        for i in contenus:
            Utils.statdb.contenu.update({"id": i}, {"$push": {"evals": max_id_eval + 1}})

    ## Récupérer les évaluations d'un auteur
    #   @param auteur le nom de l'auteur
    #   @return les évaluations de l'auteur

    @staticmethod
    def get_evaluations(auteur, id=0):
        if id == 0:
            return Utils.statdb.evaluation.find({"auteur": auteur})
        else:
            return Utils.statdb.evaluation.find({"auteur": auteur} and {"idEval": id})

    ## Modifier un contenu
    #   @param idContenu contenu à modifier
    #   @param auteur l'utilisateur désirant modifier la question
    #   @param isEnCours Vrai si modification d'un contenu en cours
    @staticmethod
    def modifier_contenu(idContenu, auteur, enCours=False):

        # le chemin du fichier restera tjrs le même
        contenu = Utils.statdb.contenu.find_one({"id": int(idContenu)})
        Utils.modification_fichier(Utils.chemin + contenu["fichier"], idContenu, auteur, not enCours)

    @staticmethod
    ## affichage d'un contenu de fichier
    # @param chemin le fichier à afficher
    def affichage(chemin):
        with open(chemin, "r") as fichier:
            texte = fichier.read()

        texte = texte.strip()
        print("Le fichier (v", int(texte[-1]), ") : " + texte)
        return texte

    @staticmethod
    ## consulter un contenu
    # @param id l'id du contenu à consulter
    # @param auteur l'auteur désirant consulter le contenu
    def consulter_un_contenu(id, auteur):
        # Créer un répertoire au nom de l'auteur (ou l'utiliser si il existe déja)
        auteur = auteur.lower().replace(" ", "_")
        repertoire_auteur = Utils.chemin + auteur + "/"
        Path(repertoire_auteur).mkdir(parents=True, exist_ok=True)
        # Copier le fichier lié au contenu dans ce répertoire
        chemin = Utils.get_chemin_du_contenu(id)
        nom_fichier = chemin[chemin.rfind("/") + 1:]
        os.system("cp '" + chemin + "'  '" + repertoire_auteur + nom_fichier + "'")
        Utils.affichage(chemin)

    ## détruire un fichier temporaire
    @staticmethod
    def detruire_fichier_tmp(id, auteur):
        auteur = auteur.lower().replace(" ", "_") + "/"
        chemin = Utils.get_chemin_du_contenu(id)
        nom_fichier = chemin[chemin.rfind("/") + 1:]
        os.system("rm '" + Utils.chemin + auteur + nom_fichier + "'")

    ## modification (directe ici) d'un fichier
    # @param chemin chemin d'acces au contenu
    # @param id id du contenu
    # @param auteur auteur qui fait la modification du contenu
    # @param modifPremiere permet de déterminer si on est sur une premiere modification du contenu ou si on remodifie un contenu en cours

    @staticmethod
    def modification_fichier(chemin, idContenu, auteur, modifPremiere=True):

        #print("Le chemin du fichier est : ", chemin)

        # on récupère la fin du fichier (la version)
        fin = Utils.affichage(chemin)
        version = int(fin[-1])
        if version == 1:
            ecart = 2
        else:
            ecart = 1
        repertoire = chemin[:chemin.rfind("/") + 1]
        nomFichier = chemin[chemin.rfind("/") + 1:]

        #print("nomFichier :", nomFichier)
        #print("chemin :", chemin)
        # exit(0)

        if modifPremiere:
            # on créée une nouvelle branche et on va dedans
            nomBranche = "version" + str(version + 1)
            #print("Je veux la branche", nomBranche)
            UtilsGit.embranchement(repertoire, nomBranche)
            UtilsGit.changerVersionBranche(repertoire, nomBranche)
        # si on est pas en "modifPremiere" on est déja dans la bonne branche.

        # 2. on remplace le dernier caractère du fichier en l'incrémentant de 1
        with open(chemin, "r+") as fichier:
            # pos = fichier.tell()
            # pos = pos - 1
            fichier.seek(0, 2)
            # seek to end of file; f.seek(0, os.SEEK_END) is legal

            fichier.seek(fichier.tell() - ecart, 0)
            fichier.truncate()
            fichier.write(str(version + 1))
            # fichier.seek(0, 2)
            # print("pront:", fichier.read())
            fichier.close()
        Utils.affichage(chemin)
        UtilsGit.commit(repertoire, nomFichier, "modification")

        # si on est dans le cas d'une modification 1ere d'un contenu, on l'entre en bd (sinon il y est déja)
        if modifPremiere:
            max_id_encours = Utils.get_max_id_contenus_en_cours()
            if max_id_encours is None:
                max_id_encours = 0
            Utils.statdb.encours.insert_one(
                {"idContenuEC": max_id_encours + 1, "date": datetime.now().strftime("%d/%m/%y"), "auteur": auteur,
                 "reference": int(idContenu), "version": version, "branche": nomBranche})

    ## Mettre à jour un contenu (branche principale)
    @staticmethod
    def maj_contenu(chemin, idContenu, nomBranche):

        repertoire = chemin[:chemin.rfind("/") + 1]
        fin = Utils.affichage(chemin)
        version = int(fin[-1])

        # Mise à jour du répertoire git
        idCommit = UtilsGit.mergeVersion(repertoire, nomBranche)

        # Mise à jour des métadonnées - ok
        version_en_cours = Utils.statdb.contenu.find_one({"id": idContenu})["versionEnCours"]
        Utils.statdb.contenu.update_one({"id": int(idContenu)}, {"$push": {"historique": version_en_cours}})
        Utils.statdb.contenu.update_one({"id": int(idContenu)}, {
            "$set": {"versionEnCours": {"numero": version, "date": datetime.now().strftime("%d/%m/%y"),
                                        "popularite": 0, "commit": idCommit}}})

    ## Supprimer une évaluation (définitif)
    # @param idEval l'id de l'évaluation à supprimer
    @staticmethod
    def supprimer_evaluation(idEval):
        Utils.statdb.evaluation.delete_one({"idEval": idEval})

    ## Fonction pour savoir si un contenu est favori chez qqun
    # @param idContenu
    @staticmethod
    def existe_en_tiers(idContenu):
        return Utils.statdb.utilisateur.find(
            {"tiers": {"$elemMatch": {"$elemMatch": {"contenu": idContenu}}}}).count() != 0

    ## On entre l'id d'un contenu favori pour un auteur, on veut récupérer en sortie la version associée
    # @param idContenu
    @staticmethod
    def get_version_tiers_avec_id(idContenu, auteur):
        tiers = (Utils.statdb.utilisateur.find_one({"nom": auteur})["tiers"])
        for i in range(len(tiers)):
            tmp = tiers[i]
            if int(tmp[0]["contenu"]) == idContenu:
                return int(tmp[1]["version"])
        return 0

    ## Suppression définitive d'un contenu
    @staticmethod
    def aneantir_contenu(idContenu):
        Utils.statdb.contenu.delete_one({"id": idContenu})

    ## permet de récupérer tous les contenus en cours pour un auteur
    # @param auteur le nom de l'auteur

    @staticmethod
    def get_contenus_en_cours(auteur):
        return Utils.statdb.encours.find({"auteur": auteur})

    ## permet de récupérer le contenu originel associé à un contenu en cours
    # @param idContenuEnCours l'id du contenu en cours de modification

    @staticmethod
    def get_id_contenu_associe_a_un_en_cours(idContenuEnCours):
        #print("L'id en cours que j'ai est", idContenuEnCours, " - ", type(idContenuEnCours))
        return Utils.statdb.encours.find_one({"idContenuEC": int(idContenuEnCours)})["reference"]

    @staticmethod
    def get_id_contenu_associe_a_une_publication(idDdePublication):
        # print("L'id en cours que j'ai est", idContenuEnCours, " - ", type(idContenuEnCours))
        return Utils.statdb.demandesPub.find_one({"id": int(idDdePublication)})["idContenu"]

    @staticmethod
    def get_id_contenu_en_cours_associe_a_une_publication(idDdePublication):
        #print("L'id en cours que j'ai est", idDdePublication, " - ", type(idDdePublication))
        return Utils.statdb.demandesPub.find_one({"id": int(idDdePublication)})["idContenuEnCours"]

    ## récupérer le chemin d'un fichier associé à un contenu
    # @param idContenu l'id du contenu

    @staticmethod
    def get_chemin_du_contenu(idContenu):
        return Utils.chemin + Utils.statdb.contenu.find_one({"id": idContenu})["fichier"]

    ## permet de changer de version pour un contenu (modification de la branche / du commit en fonction de la situation)
    # @param idContenu le contenu concerné
    # @param version la version du contenu désirée
    # @param nomBranche changement direct de branche -> utile pour naviguer parmi les versions "en cours"
    @staticmethod
    def changer_version(idContenu, version, nomBranche=""):

        # on recupere le chemin du fichier associé au contenu (dans les 2 cas)
        chemin = Utils.get_chemin_du_contenu(idContenu)
        repertoire = chemin[:chemin.rfind("/") + 1]

        if version != 0:
            # on change le repertoire récupéré ci dessus dans la bonne version (si on travaille avec une version)
            idCommit = Utils.get_commit_pour_une_version(idContenu, version)
            UtilsGit.changerVersionCommit(repertoire, idCommit)
        else:
            UtilsGit.changerVersionBranche(repertoire, nomBranche)

    ## supprimer un contenu en cours
    # @param idContenuEnCours id du contenu a supprimer
    @staticmethod
    def supprimer_contenu_en_cours(idContenuEnCours):
        Utils.statdb.encours.delete_one({"idContenuEC": idContenuEnCours})

    ## récupérer l'auteur initial d'un contenu en cours (peut être l'auteur effectuant la modification ou non)
    # @param idContenuEnCours l'id du contenu en cours de modification
    @staticmethod
    def get_auteur_contenu_en_cours(idContenuEnCours):
        id_contenu = Utils.get_id_contenu_associe_a_un_en_cours(idContenuEnCours)
        return Utils.statdb.contenu.find_one({"id": id_contenu})["auteur"]

    ## récupérer la branche d'un contenu en cours
    # @param idContenuEnCours l'id du contenu en cours de modification

    @staticmethod
    def get_branche_contenu_en_cours(idContenuEnCours):
        # id_contenu = Utils.getIdContenuAssocieAUnEnCours(idContenuEnCours)
        return Utils.statdb.encours.find_one({"idContenuEC": idContenuEnCours})["branche"]

    ## A NE JAMAIS UTILISER SANS REFLECHIR
    # Suppression integrale de la base de données & recreation des 3 auteurs
    @staticmethod
    def tout_peter():
        Utils.statdb.contenu.delete_many({})
        Utils.statdb.utilisateur.delete_many({})
        Utils.statdb.evaluation.delete_many({})
        Utils.statdb.encours.delete_many({})
        Utils.statdb.demandesPub.delete_many({})
        os.system("rm -rf dossierContenu*/")
        os.system("rm -rf auteur_*/")
        Utils.ajouter_utilisateur("Auteur 1")
        Utils.ajouter_utilisateur("Auteur 2")
        Utils.ajouter_utilisateur("Auteur 3")

    ## creer un contenu à partir d'un contenu en cours
    @staticmethod
    def creer_nouveau_contenu(chemin, auteurEnCours, idContenuEnCours):
        #print("chemin : ", chemin)

        # hard codé pour l'exemple, sera adaptatif dans le futur
        max_id = 3

        dossierFichier = "dossierContenu" + str(max_id) + "/"
        cheminFichier = Utils.chemin + dossierFichier
        nomFichier = chemin[chemin.rfind("/") + 1:]

        #print("Chemin fichier : ", cheminFichier)
        #print("Dossier Fichier : ", dossierFichier)
        #print("NomFichier :", nomFichier)
        Path(cheminFichier).mkdir(parents=True, exist_ok=True)
        os.system("cp " + chemin + " " + cheminFichier)
        os.system("mv " + cheminFichier + nomFichier + " " + cheminFichier + "Contenu" + str(max_id))
        # exit(0)
        commit = UtilsGit.commit(cheminFichier, "Contenu" + str(max_id), "Version 1")

        Utils.statdb.contenu.insert_one(
            {"id": max_id + 1, "auteur": auteurEnCours, "note": randrange(0, 10, 1),
             "fichier": "dossierContenu" + str(max_id) + "/Contenu" + str(max_id),
             "versionEnCours": {"numero": 1, "date": datetime.now().strftime("%d/%m/%y"),
                                "popularite": 0, "commit": commit},
             "tags_fixes": {"1": "tag1", "2": "tag1", "3": "tag1", "4": "tag1",
                            "5": "tag1"},
             "tags": {"1": "tag1", "2": "tag1", "3": "tag1", "4": "tag1", "5": "tag1"}})
        print("Le contenu a bien été ajouté pour l'utilisateur ", auteurEnCours)

        Utils.supprimer_contenu_en_cours(idContenuEnCours)

    ## fonction qui permet de savoir si un contenu fait partie des tiers d'un auteur
    # On fraude un peu en utilisant  getVersionFavoriById -> si la sortie est à 0 ben le contenu n'est pas un favori
    @staticmethod
    def is_tiers_auteur(auteur, idContenu):
        return Utils.get_version_tiers_avec_id(idContenu, auteur) != 0

    ## créer une demande de Publication à l'auteur d'origine
    # @param idContenuEnCours la modification à soumettre
    # @param idContenu le contenu de base
    # @param auteurEnCours l'auteur de la modification
    # @param auteur l'auteur du contenu d'origine
    # ATTENTION, IL FAUDRA SUREMENT INSERER UN ID POUR PLUS DE FACILITE A MANIPULER
    @staticmethod
    def demande_publication(idContenu, idContenuEnCours, auteurEnCours, auteur):
        max_id = Utils.get_max_id_demandes_de_publication()
        if max_id is None:
            max_id = 0
        Utils.statdb.demandesPub.insert_one(
            {"id": max_id + 1, "date": datetime.now().strftime("%d/%m/%y"), "auteurDemande": auteurEnCours,
             "auteurContenu": auteur,
             "idContenu": idContenu, "idContenuEnCours": idContenuEnCours, "etat": "enCours"})

    ## recuperer toutes les demandes de publication en cours pour un auteur
    # @param auteur l'auteur concerné
    # @param emises si true -> publications envoyées, si False, demandes recues
    @staticmethod
    def get_demandes_pub(auteur, emises=True):
        if emises:
            return Utils.statdb.demandesPub.find({"auteurDemande": auteur})
        else:
            return Utils.statdb.demandesPub.find({"auteurContenu": auteur})

    ## permet de recuperer un commit particulier lié avec une version donnee
    # On doit pouvoir récupérer également le commit de la version de base
    # @param idContenu l'id du contenu manipulé
    # @param version le numéro de version désiré
    @staticmethod
    def get_commit_pour_une_version(idContenu, version):
        print("Je veux récupérer le commit associé à la version ", version, "pour le contenu ", idContenu)



        if version == Utils.get_version_courante(idContenu):
            courante = Utils.statdb.contenu.find_one({"id": idContenu})["versionEnCours"]
            return courante["commit"]
        else:
            historique = Utils.statdb.contenu.find_one({"id": idContenu})["historique"]
            for elt in historique:
                if elt["numero"] == version:
                    return elt["commit"]

    ## permet de gerer la reponse à la publication
    # @param idPublication la publication concernée
    # @param reponse la reponse donnée (o / n)
    @staticmethod
    def gerer_demande_pub(idPublication, reponse, auteur):
        if reponse == 'o':
            print("Publication acceptée")
            id_en_cours = Utils.get_id_contenu_en_cours_associe_a_une_publication(idPublication)
            branche = Utils.get_branche_contenu_en_cours(id_en_cours)
            idContenu = Utils.get_id_contenu_associe_a_une_publication(idPublication)
            chemin = Utils.get_chemin_du_contenu(idContenu)
            Utils.maj_contenu(chemin, idContenu, branche)
            Utils.statdb.demandesPub.update_one({"id": int(idPublication)}, {"$set": {"etat": "accepté"}})
            Utils.maj_tiers(idContenu, Utils.get_version_courante(idContenu), auteur)
            Utils.supprimer_contenu_en_cours(id_en_cours)
        else:
            print("Publication refusée ", idPublication)

            # on change la valeur "acceptation" de la demande de publication
            Utils.statdb.demandesPub.update_one({"id": int(idPublication)}, {"$set": {"etat": "refus"}})
            # Utils.statdb.demandesPub.update_one({"id": 1}, {"$set": {"etat": "refus"}})

    ## mettre à jour un favori
    #   @param idContenu l'id du favori
    #   @param versionContenu la version du favori
    #   @param auteur l'auteur concerné
    #   @param refuse True = maj acceptée, False = maj refusée
    def maj_tiers(idFavori, idNouvelleVersion, auteur, refuse=False):
        if not refuse:

            # on vire le précédent
            Utils.statdb.utilisateur.update({"nom": auteur},
                                            {"$pull": {
                                                "tiers": [{"contenu": idFavori},
                                                          {"version": idNouvelleVersion - 1}]}})

            # et on ajoute la version maj
            Utils.statdb.utilisateur.update({"nom": auteur},
                                            {"$push": {
                                                "tiers": [{"contenu": idFavori}, {"version": idNouvelleVersion}]}})
        else:
            Utils.statdb.utilisateur.update({"nom": auteur},
                                            {"$push": {
                                                "majRejetees": [{"contenu": idFavori, "version": idNouvelleVersion}]}})

    ## Fonction permettant de savoir si une maj de favori a été refusée ou pas
    # @param auteur l'auteur concerné
    #   @param idContenu l'id du favori
    #   @param versionMaj la version de la mise à jour potentielle
    @staticmethod
    def is_maj_refusee(auteur, idContenu, versionMaj):
        compteur = Utils.statdb.utilisateur.find({"nom": auteur} and
                                                 {"majRejetees": {"$elemMatch": {"$elemMatch": {"contenu": idContenu,
                                                                                                "version": versionMaj}}}}).count() != 0
        #print("Compteur : ", compteur)
        return compteur

    ## Fonction globale de gestion de la recherche sur contenu
    # Très basique pour le moment elle permettra des recherches plus complexes prochainement

    @staticmethod
    def recherche_sur_contenu(action, texteRecherche):
        if action == 1:
            print("Recherche pour l'auteur ", texteRecherche)
            if Utils.presence_utilisateur(texteRecherche):
                print("J'ai", Utils.statdb.contenu.find({"auteur": texteRecherche}).count(),
                      "contenu(s) pour cet auteur")
            else:
                print("Utilisateur inconnu")
        elif action == 2:
            print("Recherche pour la date ", texteRecherche)
            print("J'ai", Utils.statdb.contenu.find({"versionEnCours.date": texteRecherche}).count(),
                  "contenu(s) pour cette date")
        elif action == 3:
            print("Recherche pour la note ", texteRecherche)
            print("J'ai", Utils.statdb.contenu.find({"note": int(texteRecherche)}).count(),
                  "contenu(s) pour cette note")
        elif action == 4:
            print("Recherche pour le tag ", texteRecherche)
            print("J'ai", Utils.statdb.contenu.find({"tags_banque_fixes": texteRecherche}).count(),
                  "contenu(s) pour ce tag (fixe)")
            print("J'ai", Utils.statdb.contenu.find({"tags_banque_libres": texteRecherche}).count(),
                  "contenu(s) pour ce tag (libre)")
        elif action == 5:
            print("Recherche pour le contenu ", texteRecherche)

    from nltk.corpus import stopwords
    from nltk.tokenize import word_tokenize
    ## Fonction test permettant de récupérer le contenu des fichiers
    @staticmethod
    def get_contenu_fichiers():
        fichier1 = "Contenu1"
        fichier2 = "Contenu2"

        chemin1 = Utils.chemin + fichier1
        chemin2 = Utils.chemin + fichier2
        with open(chemin1, "r") as fichier:
            texte = fichier.read()

        contenu1 = fichier1 + " " + texte

        with open(chemin2, "r") as fichier:
            texte = fichier.read()

        contenu2 = fichier2 + " " + texte

        french_stopwords = set(stopwords.words('french'))
        filtre_stopfr = lambda text: [token for token in texte if token.lower() not in french_stopwords]

        print(contenu1.split())
        contenu1 = filtre_stopfr(contenu1.split())
        print(contenu1)
        stringC1 = ' '.join(list(set(contenu1)))
        stringC1 = stringC1.translate(str.maketrans('', '', string.punctuation))
        stringC1 = stringC1.translate(str.maketrans('', '', string.digits))

        contenu1 = stringC1.split()

        contenu2 = filtre_stopfr(contenu2)
        print(contenu2)
        stringC2 = ' '.join(list(set(contenu1)))
        stringC1 = stringC1.translate(str.maketrans('', '', string.punctuation))
        stringC1 = stringC1.translate(str.maketrans('', '', string.digits))

        contenu1 = stringC1.split()

    ## Fonction permettant de détecter si les auteurs sont créés, et les créer si besoin
    # (exclusive squelette)
    @staticmethod
    def creer_auteur_si_besoin():
        nmbreAuteurs = Utils.statdb.utilisateur.find({}).count()
        if not nmbreAuteurs:
            Utils.ajouter_utilisateur("Auteur 1")
            Utils.ajouter_utilisateur("Auteur 2")
            Utils.ajouter_utilisateur("Auteur 3")

    ## Supprimer une demande de publication
    # @param idPub l'id de la demande de publication
    @staticmethod
    def supprimer_publication(idPub):
        Utils.statdb.demandesPub.delete_one({"id": idPub})

    ## Récupérer l'auteur d'une demande de publication
    # @param idPub l'id de la demande de publication
    @staticmethod
    def get_auteur_demande_publication(idPub):
        return Utils.statdb.demandesPub.find_one({"id": idPub})["auteurDemande"]

    ## Fonction qui permet de savoir si un contenu en cours à fait l'objet d'une demande de publication
    # @param idEnCours l'id du contenu en cours
    @staticmethod
    def is_en_cours_demande(idEnCours, auteur):
        return Utils.statdb.demandesPub.find({"id": idEnCours} and {"auteurDemande": auteur}).count() != 0
